/* This is "complex.c", written to accompany Introduction to
   Scientific Programming by Joseph L. Zachary. */

/* This file implements the complex number ADT whose interface
   appears in "complex.h". */


#include <stdio.h>
#include <math.h>
#include "complex.h"



/* Returns the complex number whose real part is "real" and whose
   imaginary part is "imaginary". */

complex make (double real, double imaginary)
{
  complex c;
  c.real = real;
  c.imag = imaginary;
  return(c);
}



/* Reads a complex number from the keyboard and returns it.  The
   number should be entered in the form a+bi or a-bi, where a and
   b stand for arbitrary doubles. */

complex read (void)
{
  complex c;
  scanf("%lf%lfi", &c.real, &c.imag);
  return(c);
}



/* Writes "c" to the display in the form a+bi or a-bi, where a and
   b stand for arbitrary doubles. */

void write (complex c)
{
  printf("%.10g%+.10gi", c.real, c.imag);
}



/* Returns c1+c2. */

complex add (complex c1, complex c2)
{
  complex result;
  result.real = c1.real + c2.real;
  result.imag = c1.imag + c2.imag;
  return(result);
}



/* Returns c1-c2. */

complex sub (complex c1, complex c2)
{
  complex result;
  result.real = c1.real - c2.real;
  result.imag = c1.imag - c2.imag;
  return(result);
}



/* Returns c1*c2. */

complex mul (complex c1, complex c2)
{
  complex result;
  result.real = c1.real*c2.real - c1.imag*c2.imag;
  result.imag = c1.real*c2.imag + c1.imag*c2.real;
  return(result);
}



/* Returns c1/c2. */

complex div (complex c1, complex c2)
{
  complex result;
  double denom;

  denom = c2.real*c2.real + c2.imag*c2.imag;
  result.real = (c1.real*c2.real + c1.imag*c2.imag) /
                   denom;
  result.imag = (c1.imag*c2.real - c1.real*c2.imag) /
                   denom;

  return(result);
}



/* Returns |c|. */

double absval (complex c) {
  return(sqrt(c.real*c.real + c.imag*c.imag));
}
