/* This is "errors.c", written to accompany Introduction to
   Scientific Programming by Joseph L. Zachary .*/

/* This program illustrates the behavior of C under certain error
   conditions.  You sould use it to explore the behavior of your
   particular version of C so that you can better understand the
   source of errors in your programs when they occur. */


#include <stdio.h>
#include <math.h>
#include <float.h>
#include <limits.h>



/* We simply print the results of a sequence of erroneous calculations. */

void main (void) {

  /* Square the maximum double to cause overflow; square the minimum
     double to cause underflow; divided by zero. */

  printf("Floating-point arithmetic:\n");
  printf("  %g * %g = %g\n", DBL_MAX, DBL_MAX, DBL_MAX * DBL_MAX);
  printf("  %g * %g = %g\n", DBL_MIN, DBL_MIN, DBL_MIN * DBL_MIN);
  printf("  1.0 / 0.0 = %g\n\n", 1.0/0.0);
  

  /* Take the square root of -1; take the arccos of 2; take the log of
     0. */

  printf("Built-in functions:\n");
  printf("  sqrt(%g) = %g\n", -1.0, sqrt(-1.0));
  printf("   acos(%g) = %g\n", 2.0, acos(2.0));
  printf("    log(%g) = %g\n\n", 0.0, log(0.0));


  /* Add one to the maximum integer to cause overflow; subtract one
     from the minimum integer to cause overflow; divided by zero.
     Integer division by zero is the only one of the nine errors in
     this program that will cause the program to crash, which is why
     we do it last. */

  printf("Integer arithmetic:\n");
  printf("  %d + %d = %d\n", INT_MAX, 1, INT_MAX+1);
  printf("  %d - %d = %d\n", INT_MIN, 1, INT_MIN-1);
  printf("  1 / 0 = ");
  fflush(stdout);
  printf("%d\n", 1/0);

}
